//
//  Copyright (C) 2011 Tom Beckmann
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

using Gtk;
using Granite;
using SQLHeavy;
using Gee;
using Gdk;

using Footnote.Widgets;

namespace Footnote{
	
	
	/*
	 * Store for default settings in d-conf
	 */
	public class FootnoteSettings : Granite.Services.Settings{
		
		public int window_width  {get; set;}
		public int window_height {get; set;}
		
		public FootnoteSettings (){
			base ("org.elementary.Footnote");
		}
	}
	
	
	
	
	/**
	 * the app and everything else
	 **/
	public class FootnoteApp : Granite.Application{

		public struct Selected {
			public Category category;
			public Note note;
		}
		
		public Gtk.Window mainwindow;
		public Granite.Widgets.SearchBar search;
		public ListStore topic_container;
		public SQLHeavy.Database db;
		public Transaction trans;
		public ArrayList<Category> categories;
		public Viewport note_view;
		public ToolButton add_button;
		public ToolButton remove_button;
		public ToolButton edit_button;
		public Selected selected;
		public bool showing_search;
		public Category cat;
		public Adjustment VAdj;
		
		public FootnoteSettings settings;
		
        public Gtk.ActionGroup main_actions;
        Gtk.UIManager ui;
		
		public int year;
		public int month;
		
		public Cairo.ImageSurface close;
		public Cairo.ImageSurface share;
		
		public bool prevent_apply; //helper for the toggle buttons
		
		construct{
			program_name = "Footnote";
			exec_name = "footnote";
			
			build_data_dir = Constants.DATADIR;
			build_pkg_data_dir = Constants.PKGDATADIR;
			build_release_name = Constants.RELEASE_NAME;
			build_version = Constants.VERSION;
			build_version_info = Constants.VERSION_INFO;
			
			app_years = "2011";
			app_icon = "";
			app_launcher = "footnote.desktop";
			application_id = "net.launchpad.Footnote";
			
			main_url = "https://code.launchpad.net/footnote";
			bug_url = "https://bugs.launchpad.net/footnote";
			help_url = "https://code.launchpad.net/footnote";
			translate_url = "https://translations.launchpad.net/footnote";
			
			about_authors = {"Tom Beckmann <tombeckmann@online.de>"};
			about_documenters = {"Tom Beckmann <tombeckmann@online.de>"};
			about_artists = {"Harvey Cabaguio"};
			about_comments = "Development release, not all features implemented";
			about_translators = "";
			about_license_type = License.GPL_3_0;
		}
		
		const string ui_string = """
            <ui>
            <popup name="MenuItemTool">
                <menuitem name="Quit" action="Quit"/>
                <menuitem name="New Note" action="NewNote"/>
                <menuitem name="New Notebook" action="NewNotebook"/>
                <menuitem name="Delete Note" action="DeleteNote"/>
                <menuitem name="Set Bold" action="SetBold"/>
                <menuitem name="Strike Through" action="StrikeThrough"/>
                <menuitem name="Set Italic" action="SetItalic"/>
            </popup>
            </ui>
        """;

		public FootnoteApp (){
			month = -1;
			year = 2012;
			
			close = new Cairo.ImageSurface.from_png (build_data_dir+"/footnote/note-close.png");
			share = new Cairo.ImageSurface.from_png (build_data_dir+"/footnote/note-share.png");
			
			mainwindow = new Gtk.Window ();
			mainwindow.set_default_size (640,480);
			
			categories = new ArrayList<Category> ();
			set_actions ();
			
			selected = {null, null};
			
			build ();
			
			create_database ();
			
			try{
				for  (var res=db.execute ("SELECT name,color,id FROM 'categories'");!res.finished;res.next ()){
					new Category.from_existing  (this, res.fetch_string (0), (uint32)res.fetch_int64 (1), res.fetch_int(2));
				}
			}catch (SQLHeavy.Error e){stderr.printf ("Error: %s\n", e.message);}
			
			if  (categories.size > 0){
				selected.category = categories.get (0);
				note_view.child = selected.category.view;
				note_view.show();
				selected.category.view.update_view ();
			}else{
				this.main_actions.get_action ("NewNote").sensitive = false;
			}
			
			
			settings = new FootnoteSettings ();
			
			if (settings.window_width != 0 && settings.window_height != 0)
				this.mainwindow.resize (settings.window_width, settings.window_height);
			
			mainwindow.destroy.connect ( () => {
				settings.window_width  = mainwindow.get_allocated_width ();
				settings.window_height = mainwindow.get_allocated_height ();
				Gtk.main_quit ();
			});
			
		}
		
		private void build (){
			
			var vmain = new VBox (false, 0);

			var tb = new Toolbar ();
			vmain.pack_start (tb, false, true, 0);
			tb.get_style_context  ().add_class  (STYLE_CLASS_PRIMARY_TOOLBAR);
			
			var separat = new ToolItem ();
			separat.set_expand (true);
			ToolItem search_item = new ToolItem ();
			search = new Granite.Widgets.SearchBar (_("Type to Search..."));
			search.key_release_event.connect (() => {
				on_search_changed ();
				return false;
			});
			search.icon_press.connect ((entry, event) => {
				if (entry == Gtk.EntryIconPosition.SECONDARY)
					on_search_changed ();
			});
			search_item.child = search;

			Menu menu = new Menu ();
			Granite.Widgets.AppMenu appmenu = create_appmenu(menu);
			
			var add_new = main_actions.get_action ("NewNote").create_tool_item () as ToolButton;
			var sep2    = new SeparatorToolItem ();
			
			tb.insert (add_new,     0);
			tb.insert (sep2,        1);
			tb.insert (separat,     2);
			tb.insert (search_item, 3);
			tb.insert (appmenu,     4);
			
			var hmain = new Paned (Orientation.HORIZONTAL);
			vmain.pack_start (hmain);
			
			var left_box = new VBox (false, 0);
			var scrolled = new ScrolledWindow (null, null);
			scrolled.width_request = 200;
			topic_container = new ListStore (2, typeof(Gdk.Pixbuf), typeof (string));
			var topic_tree = new TreeView.with_model (topic_container);
			topic_tree.headers_visible = false;
			topic_tree.cursor_changed.connect ( ()=>{
				note_view.remove(selected.category.view);

				TreePath path;
				topic_tree.get_cursor (out path, null);

				var cat = int.parse (path.to_string ());
				selected.category = categories.get (cat);

				note_view.child = selected.category.view;
				selected.category.view.show ();
				selected.category.view.update_view ();
				
				selected.category.modify (1);
			});
			topic_tree.row_activated.connect ( ()=> {
			    selected.category.modify ();
			});
			var cell1 = new CellRendererPixbuf ();
			cell1.set_padding (12, 8);
			topic_tree.insert_column_with_attributes (-1, "", cell1, "pixbuf", 0);
			var cell2 = new CellRendererText ();
			cell2.set_padding (0, 8);
			topic_tree.insert_column_with_attributes (-1, "", cell2, "markup", 1);
			scrolled.add (topic_tree);
			left_box.pack_start (scrolled, true, true, 0);
			
			var buttons = new Toolbar ();
			buttons.icon_size = 1;
			add_button = new ToolButton (new Image.from_icon_name ("list-add-symbolic", IconSize.MENU),"");
			add_button.clicked.connect ( ()=>{
				new Category.add (this);
			});
			edit_button = new ToolButton (new Image.from_icon_name ("folder-documents-symbolic", IconSize.MENU), "");
			edit_button.clicked.connect ( ()=>{
			    selected.category.edit (this);
			});
			remove_button = new ToolButton (new Image.from_icon_name ("list-remove-symbolic", IconSize.MENU),"");
			remove_button.clicked.connect ( ()=>{
				selected.category.del (this);
			});
			buttons.get_style_context ().add_class ("inline-toolbar");
			buttons.insert (add_button, 0);
			buttons.insert (remove_button, 1);
			buttons.insert(edit_button, 2);
			left_box.pack_start (buttons, false, false, 0);
			hmain.pack1 (left_box, false, false);
			
			
			//right side
			var left       = new Box (Orientation.VERTICAL, 0);
			var scrolled2  = new ScrolledWindow (null, null);
			note_view      = new Viewport (null,null);
			var topbar     = new Toolbar ();
			
			topbar.set_icon_size (IconSize.MENU);
			topbar.toolbar_style = ToolbarStyle.TEXT;
			topbar.get_style_context ().add_class ("secondary-toolbar");
			
			var years_item = new ToolItem ();
			var years      = new ComboBoxText ();
			years.append ("2012", "2012");
			years.append ("2011", "2011");
			years.active_id = "2012";
			years_item.add (years);
			years_item.margin_right = 10;
			
			var all   = new RadioToolButton (null);
			all.label = _("All");
			var jan   = new RadioToolButton.from_widget (all);
			jan.label = _("January");
			var feb   = new RadioToolButton.from_widget (all);
			feb.label = _("February");
			var mar   = new RadioToolButton.from_widget (all);
			mar.label = _("March");
			var apr   = new RadioToolButton.from_widget (all);
			apr.label = _("April");
			var may   = new RadioToolButton.from_widget (all);
			may.label = _("May");
			var jun   = new RadioToolButton.from_widget (all);
			jun.label = _("June");
			var jul   = new RadioToolButton.from_widget (all);
			jul.label = _("July");
			var aug   = new RadioToolButton.from_widget (all);
			aug.label = _("August");
			var sep   = new RadioToolButton.from_widget (all);
			sep.label = _("September");
			var oct   = new RadioToolButton.from_widget (all);
			oct.label = _("October");
			var nov   = new RadioToolButton.from_widget (all);
			nov.label = _("November");
			var dec   = new RadioToolButton.from_widget (all);
			dec.label = _("December");
			
			topbar.child_set (all, "homogeneous", false);
			topbar.child_set (jan, "homogeneous", false);
			topbar.child_set (feb, "homogeneous", false);
			topbar.child_set (mar, "homogeneous", false);
			topbar.child_set (apr, "homogeneous", false);
			topbar.child_set (may, "homogeneous", false);
			topbar.child_set (jun, "homogeneous", false);
			topbar.child_set (jul, "homogeneous", false);
			topbar.child_set (aug, "homogeneous", false);
			topbar.child_set (sep, "homogeneous", false);
			topbar.child_set (oct, "homogeneous", false);
			topbar.child_set (nov, "homogeneous", false);
			topbar.child_set (dec, "homogeneous", false);
			
			topbar.insert (years_item, 0);
			topbar.insert (all,        1);
			topbar.insert (jan,        2);
			topbar.insert (feb,        3);
			topbar.insert (mar,        4);
			topbar.insert (apr,        5);
			topbar.insert (may,        6);
			topbar.insert (jun,        7);
			topbar.insert (jul,        8);
			topbar.insert (aug,        9);
			topbar.insert (sep,        10);
			topbar.insert (oct,        11);
			topbar.insert (nov,        12);
			topbar.insert (dec,        13);
			
			left.pack_start (topbar, false, true, 0);
			left.pack_start (scrolled2);
			
			scrolled2.add (note_view);
			hmain.pack2 (left, true, true);
			
			
			all.toggled.connect ( () => {if (all.active)month = -1;selected.category.view.update_view ();});
			jan.toggled.connect ( () => {if (jan.active)month = 1;selected.category.view.update_view ();});
			feb.toggled.connect ( () => {if (feb.active)month = 2;selected.category.view.update_view ();});
			mar.toggled.connect ( () => {if (mar.active)month = 3;selected.category.view.update_view ();});
			apr.toggled.connect ( () => {if (apr.active)month = 4;selected.category.view.update_view ();});
			may.toggled.connect ( () => {if (may.active)month = 5;selected.category.view.update_view ();});
			jun.toggled.connect ( () => {if (jun.active)month = 6;selected.category.view.update_view ();});
			jul.toggled.connect ( () => {if (jul.active)month = 7;selected.category.view.update_view ();});
			aug.toggled.connect ( () => {if (aug.active)month = 8;selected.category.view.update_view ();});
			sep.toggled.connect ( () => {if (sep.active)month = 9;selected.category.view.update_view ();});
			oct.toggled.connect ( () => {if (oct.active)month = 10;selected.category.view.update_view ();});
			nov.toggled.connect ( () => {if (nov.active)month = 11;selected.category.view.update_view ();});
			dec.toggled.connect ( () => {if (dec.active)month = 12;selected.category.view.update_view ();});
			
			years.changed.connect ( () => {
				this.year = int.parse (years.active_id);
				selected.category.view.update_view ();
			});
			
			this.VAdj = note_view.get_vadjustment ();
			
			mainwindow.add (vmain);
			mainwindow.show_all ();
			
			TargetEntry e = {"footnote_note", 0, 0};
			topic_tree.enable_model_drag_dest ({e}, DragAction.MOVE);
			topic_tree.drag_data_received.connect ( (widget, ctx, x, y, sel, info, timestamp) => {
				TreePath path; TreeViewDropPosition pos;
				var drop_info = topic_tree.get_dest_row_at_pos (x, y, out path, out pos);
				if (drop_info){
					selected.category.notes.remove (selected.note);
					selected.category.view.remove_note (selected.note);
					
					var dest_cat = categories.get (int.parse (path.to_string ()));
					dest_cat.notes.add (selected.note);
					dest_cat.view.pack_start (selected.note);
					selected.note.category = dest_cat;
					
					selected.category.set_count_field (this, selected.category.count-1);
					selected.note.category.set_count_field (this, selected.note.category.count+1);
					
					selected.category.view.update_view ();
					try{
						var q = db.prepare ("UPDATE 'notes' SET category=:cat WHERE rowid=:id");
						q.set_int (":cat", selected.note.category.id);
						q.set_int(":id", selected.note.id);
						q.execute();
					}catch(SQLHeavy.Error e){
						stderr.printf("Error: %s\n", e.message);
					}
					Gtk.drag_finish (ctx, true, false, timestamp);
					return;
				}
				Gtk.drag_finish (ctx, false, false, timestamp);
			});
		}

		private void on_search_changed () {
			foreach (Category c in categories) {
				c.view.filter = search.text;
			}
			selected.category.view.update_view ();
		}
		
		public void set_view_to (Note n){
            var _notes = n.category.notes;
            for (int i=0;i<_notes.size;i++) {
    		    if (_notes[i] == n) {
    		        var last_note = _notes[i-1];
    		        // 222 is the size of a new note. 15 is the size of the border under the note
    		        this.VAdj.set_upper (this.VAdj.upper + 222 + 15);
    		        // it scrolls till last note plus the the size of a new note
                    this.VAdj.set_value (last_note.y + last_note.height + 222);
                    break;
                }
    		}
		}
		
		private void set_actions () {
		    main_actions = new Gtk.ActionGroup ("MainActionGroup"); /* Actions and UIManager */
            main_actions.set_translation_domain ("footnote");
            main_actions.add_actions (main_entries, mainwindow);
            //main_actions.add_toggle_actions (toggle_entries, mainwindow);
            
            ui = new Gtk.UIManager ();

            try {
                ui.add_ui_from_string (ui_string, -1);
            }
            catch(GLib.Error e) {
                error ("Couldn't load the UI: %s", e.message);
            }

            Gtk.AccelGroup accel_group = ui.get_accel_group();
            mainwindow.add_accel_group (accel_group);

            ui.insert_action_group (main_actions, 0);
            ui.ensure_update ();
		}
		
		private void create_database (){
			var db_file = GLib.File.new_for_path (Environment.get_user_data_dir ()+"/footnote/notes.db");
			var need_create = !db_file.query_exists ();
			try {
				db = new SQLHeavy.Database (db_file.get_path (),SQLHeavy.FileMode.READ|SQLHeavy.FileMode.WRITE|SQLHeavy.FileMode.CREATE);
				if (need_create) {
					db.execute ("CREATE TABLE categories (name TEXT, color INTEGER, id INTEGER PRIMARY KEY AUTOINCREMENT)");
					db.execute ("CREATE VIRTUAL TABLE notes USING fts3(title TEXT, content TEXT, category INT, tags TEXT, date TEXT)");
				}
			}catch  (SQLHeavy.Error err) {stderr.printf ("Error: %s\n", err.message);}
		}
		
		static const Gtk.ActionEntry[] main_entries = {
           { "Quit", Gtk.Stock.QUIT,
          /* label, accelerator */       N_("Quit"), "<Control>q",
          /* tooltip */                  N_("Quit"),
                                         Gtk.main_quit },
            
           { "NewNote", Gtk.Stock.NEW,
          /* label, accelerator */       N_("New Note"), "<Control>t",
          /* tooltip */                  N_("Create a new note"),
                                         action_new_note },
                                         
           { "NewNotebook", Gtk.Stock.ADD,
          /* label, accelerator */       N_("New Notebook"), "<Control>n",
          /* tooltip */                  N_("Create a new notebook"),
                                         action_new_notebook },
                                         
           { "DeleteNote", Gtk.Stock.CLOSE,
          /* label, accelerator */       N_("Delete Note"), "<Control>w",
          /* tooltip */                  N_("Move the note into the trash"),
                                         action_delete_note },
          /* };
        static const Gtk.ToggleActionEntry[] toggle_entries = {*/
           { "SetBold",   Gtk.Stock.BOLD,
                                         N_("Set Bold"), "<Control>b",
                                         N_("Set the selected text to bold"),
                                         action_set_bold },
          { "StrikeThrough",   Gtk.Stock.STRIKETHROUGH,
                                         N_("Strike through"), "<Control>d",
                                         N_("Strike though the selected text"),
                                         action_strike_through },
          { "SetItalic",   Gtk.Stock.ITALIC,
                                         N_("Set Italic"), "<Control>i",
                                         N_("Set the selected text to italic"),
                                         action_set_italic },
          { "Clear",       Gtk.Stock.CLEAR,
                                         N_("Clear styles"), "<Control>D",
                                         N_("Clear all styling on the selected text"),
                                         action_clear }
        };
	}
}

public Footnote.FootnoteApp nt;

public void action_new_note (){
	// FIXME: move Note creation into Category
	nt.selected.category.add_note (new Note.add (nt, nt.selected.category));
	// If there's only one note we don't need to focus on it because it's aready visibile
	if (nt.selected.category.count > 1)
    	nt.set_view_to (nt.selected.note);
}
public void action_new_notebook (){
	new Category.add (nt);
}
public void action_delete_note (){
	nt.selected.category.delete_note (nt.selected.note);
}
public void action_set_bold (){
	nt.selected.note.apply_tag ("bold");
}
public void action_strike_through (){
	nt.selected.note.apply_tag ("strikethrough");
}
public void action_set_italic (){
	nt.selected.note.apply_tag ("italic");
}
public void action_clear (){
	nt.selected.note.clear_tags ();
}

public static void main (string [] args){
	Gtk.init (ref args);
	nt = new Footnote.FootnoteApp ();
	Gtk.main ();
}
