//
//  Copyright (C) 2011 Tom Beckmann
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

using Gtk;
using Granite;
using SQLHeavy;

namespace Footnote.Widgets {
	/**
	 * Class represeting a category with its notes
	 **/
	public class Category : GLib.Object {
		public NoteView view;
		public int id;
		public string title;
		private TreeIter iter;
		public int count;
		private Gdk.Pixbuf line;
		public uint32 color;
		public Gee.ArrayList<Note> notes;
		public bool cat_collapsed;
		private FootnoteApp app;
		
		public Category.from_existing (FootnoteApp app, string title, uint32 color, int id){
			this.app = app;

			try{
				notes = new Gee.ArrayList<Note>();

				this.id = id;
				this.title = title;
				this.color = color;
				build (app);
				var q = new Query (app.db, "SELECT title,content,tags,rowid,date FROM 'notes' WHERE category = :id ORDER BY rowid ASC;");
				q.set_int (":id", this.id);
				for  (var res = q.execute ();!res.finished;res.next ()){
					Note note = new Note.from_existing (app, this, res.fetch_string (0), res.fetch_string (1), res.fetch_string (2), res.fetch_int (3), res.fetch_string (4));
					notes.add (note);
					view.add_note(note);
				}
			}catch (SQLHeavy.Error e) {
				stderr.printf ("Error: %s", e.message);
			}
			
			app.categories.add (this);
			this.set_count_field (app, notes.size);
		}
		
		public void set_count_field (FootnoteApp app, int count){
			this.count = count;
			string note_string = "";
			
			if (count == 0)
				note_string = _("No Notes");
			else if (count == 1)
				note_string = 1.to_string () + " " + _("Note");
			else
				note_string = count.to_string () + " " + _("Notes");

			app.topic_container.set (iter, 0, line, 1, "<span>"+this.title+"</span>\n"+
			    "<span style='italic'>"+note_string+"</span>");
		}
		
		public Category.add (FootnoteApp app){
			var dlg = new Granite.Widgets.PopOver ();
			var name = new Entry ();
			var ok = new Button.from_stock (Stock.ADD);
			var b = new VBox (false, 0);
			var colb = new ColorButton ();
			var namelbl = new Label (_("Name"));
			var colorlbl = new Label (_("Color"));
			namelbl.set_justify (Justification.LEFT);
			colorlbl.set_justify (Justification.LEFT);
			name.set_placeholder_text (_("Notebook name"));
			ok.set_sensitive(false);
			b.pack_start(namelbl);
			b.pack_start (name);
			b.pack_start (colorlbl);
			b.pack_start (colb);
			b.pack_start (ok);
			name.key_release_event.connect ( (event)=>{
				if (name.text != ""){
					ok.set_sensitive(true);
					if  (event.keyval == 65293){
						add_cat (app, name.text, dlg, get_color((uint8)colb.color.red, (uint8)colb.color.green, (uint8)colb.color.blue));
					}
				}else{
					ok.set_sensitive(false);
				}
				return false;
			});
			ok.clicked.connect ( ()=>{
				add_cat (app, name.text, dlg, get_color((uint8)colb.color.red, (uint8)colb.color.green, (uint8)colb.color.blue));
			});
			((Box)dlg.get_content_area ()).add (b);
			dlg.show_all ();
			dlg.move_to_widget (app.add_button);
			dlg.present ();
			dlg.run ();
			dlg.destroy ();
		}
		
		private void add_cat (FootnoteApp app, string title, Granite.Widgets.PopOver dlg, uint32 col){
			if (title != ""){
				this.app = app;
				this.notes = new Gee.ArrayList<Note>();
				this.title = title;
				this.color = col;
				app.remove_button.set_sensitive(true);
				app.main_actions.get_action ("NewNote").set_sensitive (true);
				dlg.destroy ();
				try{
					var q = app.db.prepare ("INSERT INTO 'categories' (name, color) VALUES (:name, :color);");
					q.set_string (":name", this.title);
					q.set_int64 (":color", this.color);
					q.execute ();
					this.id = (int)app.db.last_insert_id;
				}catch (SQLHeavy.Error e){stderr.printf ("Error: %s",e.message);}
				build (app);
				app.categories.add (this);
				app.note_view.remove(app.selected.category.view);
				app.selected.category = this;
				app.note_view.child = app.selected.category.view;
				app.selected.category.view.update_view();

				if (notes.size == 0) {
					this.add_note(new Note.add(app, this));
				}
			}
		}
		
		private void edit_cat (FootnoteApp app, string title, Granite.Widgets.PopOver dlg, uint8 r, uint8 g, uint8 b){
			if (title != ""){
				this.title = title;
				this.color = get_color (r,g,b);
				app.remove_button.set_sensitive(true);
				app.main_actions.get_action ("NewNote").set_sensitive (true);
				dlg.destroy ();
				try{
					var q = new Query (app.db, "UPDATE 'categories' SET name=:name, color=:color WHERE id=:id");
					q.set_string (":name", this.title);
					q.set_int64 (":color", (int64)this.color);
					q.set_int(":id", this.id);
					q.execute ();
				}catch (SQLHeavy.Error e){stderr.printf ("Error: %s",e.message);}
			    
			    this.set_count_field (app, this.count);
			    
			    line.fill (this.color);
			}
		}
		
 		public void modify (int force=0){
 		    switch (force) {
 		    case 0:
 		        if (this.cat_collapsed) {
 		            this.cat_collapsed = false;
     		        foreach (Note n in this.notes) {
 	    	            n._expand ();
 	    	        }
 	    	    }else{
 	    	        this.cat_collapsed = true;
         		    foreach (Note n in this.notes) {
 	    	            n.collapse ();
 	    	        }
 		        }
 		        break;
 		    case 1:
 		        if (this.cat_collapsed) {
     	    	    foreach (Note n in this.notes) {
 	           	        n._expand ();
 	           	    this.cat_collapsed = true;
                     }
 	    	    }
 	    	    break;
 		    case 2:
 		        this.cat_collapsed = true;
         		foreach (Note n in this.notes) {
 	    	        n.collapse ();
 	    	    }
 	    	    break;
 		    }
 		}
		
		private uint32 get_color (uint8 r, uint8 g, uint8 b){
			return (uint32)((r<<24)+(g<<16)+(b<<8)+0x00000000);
		}
		
		private double[] from_color (uint32 color){
	        uint32 r = color>>24;
	        uint32 g = color>>16;
	        uint32 bl = color>>8;
            return {(double)((uint8)r)/255.0,(double)((uint8)g)/255.0,(double)((uint8)bl)/255.0};
		}
		
		private void build_pixbuf (){
			var w = 8;
			var h = 50;
			line = new Gdk.Pixbuf (Gdk.Colorspace.RGB, false, 8, w, h);
			line.fill (this.color);
		}
		
		private void build (FootnoteApp app){
    		this.cat_collapsed = false;
			build_pixbuf ();
			app.topic_container.append (out iter);
			app.topic_container.set (iter, 0, line, 1, "<span size='small'>"+this.title+"</span>\n"+
			      "<span size='small' style='italic'>"+0.to_string ()+" Notes</span>");
			view = new NoteView(app);
		}
		
		public void edit (FootnoteApp app){
			var dlg = new Granite.Widgets.PopOver ();
			var name =  new Entry ();
			var edit = new Button.from_stock (Stock.EDIT);
			var b = new VBox (false, 0);
			var colb = new ColorButton ();
			var namelbl = new Label (_("Name"));
			var colorlbl = new Label (_("Color"));
			namelbl.set_justify (Justification.LEFT);
			colorlbl.set_justify (Justification.LEFT);
			name.set_placeholder_text (_("Notebook name"));
            name.set_text (_(this.title));
            double[] rgb = from_color(this.color);
			var CurColor = Gdk.RGBA (){red=rgb[0],green=rgb[1],blue=rgb[2]};
            colb.set_rgba(CurColor);
			b.pack_start(namelbl);
			b.pack_start (name);
			b.pack_start (colorlbl);
			b.pack_start (colb);
			b.pack_start (edit);
			name.key_release_event.connect ( (event)=>{
				if (name.text != ""){
					edit.set_sensitive(true);
					if  (event.keyval == 65293){
						edit_cat (app, name.text, dlg, (uint8)colb.color.red, (uint8)colb.color.green, (uint8)colb.color.blue);
					}
				}else{
					edit.set_sensitive(false);
				}
				return false;
			});
			edit.clicked.connect ( ()=>{
				edit_cat (app, name.text, dlg, (uint8)colb.color.red, (uint8)colb.color.green, (uint8)colb.color.blue);
			});
			((Box)dlg.get_content_area ()).add (b);
			dlg.show_all ();
			dlg.move_to_widget (app.edit_button);
			dlg.present ();
			dlg.run ();
			dlg.destroy ();
		}
		
		public void del (FootnoteApp app){
			var dlg = new Granite.Widgets.PopOver ();
			var b = new VBox (false, 0);
			var yes = new Button.from_stock (Stock.DELETE);
			
			b.pack_start (new Label (_("Delete topic with all its notes?")));
			b.pack_start (yes);
			((Box)dlg.get_content_area ()).add (b);
			
			yes.clicked.connect ( ()=>{
				try{
					Query q = new Query(app.db, "DELETE FROM 'categories' WHERE id = :id;");
					q.set_int(":id", this.id);
					q.execute();
					Query r = new Query(app.db, "DELETE FROM 'notes' WHERE category = :id;");
					r.set_int(":id", this.id);
					r.execute();
				}catch (SQLHeavy.Error e){print ("Error: %s\n", e.message);}

				app.categories.remove (this);
				app.topic_container.remove (iter);
				
				if (app.categories.size != 0){
					app.note_view.remove(app.selected.category.view);
					app.selected.category = app.categories.get (0);
					app.main_actions.get_action ("NewNote").set_sensitive (true);
					app.note_view.child = app.selected.category.view;
				}else{
					app.selected.category = null;
					app.main_actions.get_action ("NewNote").set_sensitive (false);
					app.remove_button.set_sensitive(false);
				}

				foreach (Note n in notes)
					view.remove_note(n);

				dlg.destroy ();
			});
			dlg.show_all ();
			dlg.move_to_widget (app.remove_button);
			dlg.present ();
			dlg.run ();
			dlg.destroy ();
		}

		public void add_note (Note n) {
			if (notes.size > 0)
				this.notes.insert (1, n);
			else
				this.notes.insert (0, n);
			this.view.add_note (n);
			this.set_count_field (app, notes.size);
		}

		public void delete_note (Note n) {
			this.view.remove_note (n);
			this.notes.remove (n);
			this.set_count_field (app, notes.size);
			n.del_from_db (this.app);
			this.view.update_view ();
		}
	}
}
	
