//
//  Copyright (C) 2011 Tom Beckmann
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

using Gtk;
using Granite;
using SQLHeavy;
using Gee;

namespace Footnote.Widgets {
	/**
	 * class representing a note
	 **/
	public class Note : EventBox{
		public string title;
		public string content;
		public ArrayList<string> tags;
		public Category category;
		public int id;
		public TextView title_entry;
		public TextView content_entry;
		public string date;
		public Widgets.TaggingArea tag_area;
		public EventBox close;
		FootnoteApp app;
		public int y;
		public int height;
		public int width;
		public Granite.Drawing.BufferSurface buffer;
		public Toolbar editbar;
 		private bool collapsed;
 		private HBox header;
 		private Button view_in_main;
 		public Button view_collapsed;
 		private Label datelbl;
 		private Box content_box;
		
		public int month;
		public int year;
		
		public Note.from_existing (FootnoteApp app, Category category, string title, string content, string tags, int id, string date){
			this.app = app;
			this.title = title;
			this.content = content;
			this.category = category;
			this.id = id;
			this.date = date;
			this.tags = new ArrayList<string> ();
			var parts = tags.split (",");
			for  (var i=0;i<parts.length;i++){
				this.tags.add (parts[i]);
			}
			this.build (app);
			title_entry.buffer.text = title;
			content_entry.buffer.text = content;
			
			int m = date.index_of (".");
			int y = date.index_of (".", m+1);
			this.month = int.parse (date.slice (m+1, y));
			this.year  = int.parse (date.substring (y+1));
		}
		
		private void draw_buffer (){
			int padding = 15;
			int w = this.get_allocated_width  ();
			int h = this.get_allocated_height ();
			
			buffer = new Granite.Drawing.BufferSurface (w, h);
			
			Granite.Drawing.Utilities.cairo_rounded_rectangle (buffer.context, 
			    padding, padding, w-padding*2, h-padding*2, 4);
			
			buffer.context.set_source_rgba (0.0, 0.0, 0.0, 0.5);
			buffer.context.fill_preserve ();
			buffer.exponential_blur (4);
			
			buffer.context.set_source_rgba (1.0, 1.0, 1.0, 1.0);
			buffer.context.fill ();
			
			buffer.context.set_source_surface (app.close, 0, 0);
			buffer.context.paint_with_alpha (1.0);
			
			buffer.context.set_source_surface (app.share, w-32, 0);
			buffer.context.paint_with_alpha (1.0);
		}
		
		public override bool draw (Cairo.Context ctx){
			if (buffer == null)
				draw_buffer ();
			ctx.set_source_surface (buffer.surface, 0, 0);
			ctx.paint_with_alpha (1.0);
			return base.draw (ctx);
		}
		
		private void build (FootnoteApp app){		    
			this.margin = 15;
			
		    this.size_allocate.connect ( ()=>{ 
    			Allocation alloc;
                this.get_allocation (out alloc);
                this.y = alloc.y;
                this.height = this.get_allocated_height ();
                this.width  = this.get_allocated_width  ();
				draw_buffer ();
		    });
			
			this.visible_window = false;
			this.set_resize_mode (ResizeMode.PARENT);
			var entry_box = new VBox (false,0);
			entry_box.margin = 15;
			
			datelbl = new Label ("");
			datelbl.set_markup ("<span color='#aaa'>"+this.date+"</span>");
			datelbl.margin_top = 10;
			
			var body = new VBox (false, 0);
					
			header = new HBox (false, 0);
			
		    view_in_main = new Button ();
			view_in_main.set_image (new Image.from_icon_name ("go-down-symbolic", IconSize.MENU));
            view_in_main.set_relief(Gtk.ReliefStyle.NONE);
            view_in_main.margin_left = 8;
            view_in_main.pressed.connect ( ()=> {
                modify ();
            });
            view_in_main.set_no_show_all (true);
            header.pack_start (view_in_main, false, false, 0);
			
		    view_collapsed = new Button ();
		    view_collapsed.set_image (new Image.from_icon_name ("go-up-symbolic", IconSize.MENU));
            view_collapsed.set_relief(Gtk.ReliefStyle.NONE);
            view_collapsed.margin_left = 8;
            view_collapsed.pressed.connect ( ()=> {modify ();});
            view_collapsed.set_no_show_all (true);
			header.pack_start (view_collapsed, false, true, 0); 
			
			title_entry = new TextView ();
			title_entry.wrap_mode = WrapMode.NONE;
			title_entry.modify_font (Pango.FontDescription.from_string ("bold, 15"));
			title_entry.margin = 5;
			title_entry.indent = 5;
			title_entry.focus_out_event.connect ( () => {this.on_focus_out ();return false;} );
			title_entry.focus_in_event.connect ( () => {
			if (!collapsed)
			    this.on_focus_in ();
			return false;} );
			title_entry.grab_focus ();
			title_entry.accepts_tab = false;
			TextBuffer title_buffer = title_entry.get_buffer ();
			TextIter start;
			TextIter end;
			title_buffer.get_start_iter (out start);
			title_buffer.get_end_iter (out end);
			title_buffer.changed.connect ( () => {
				title_buffer.get_start_iter (out start);
				title_buffer.get_end_iter (out end);
				string content = title_buffer.get_text (start, end, true);
				if (content[content.length-1] == '\n') {
					title_buffer.set_text(content[0:-1], content.length-1);
					content_entry.grab_focus ();
				}
			});
			
			title_entry.focus_out_event.connect ( () => {
				on_focus_out ();
				return false;
			});
			title_entry.focus_in_event.connect ( () => {
				on_focus_in ();
				return false;
			});
			title_entry.draw.connect ( (ctx) => {
				if (app.selected.note == this){
					ctx.rectangle (0, 0, 
					    title_entry.get_allocated_width (), title_entry.get_allocated_height ());
					ctx.set_source_rgba (0.0, 0.0, 0.0, 0.4);
					ctx.set_line_width (2.0);
					ctx.stroke ();
				}
				return false;
			});
			header.pack_start (title_entry, true, true, 0);
			
			//content entry
			content_box = new Box (Orientation.VERTICAL, 0);
			content_box.margin = 5;
			
			content_entry = new TextView ();
			content_entry.margin = 1;
			content_entry.indent = 5;
			content_entry.margin_top = 0;
			content_entry.height_request = 100;
			content_entry.focus_out_event.connect ( () => {
				on_focus_out ();
				return false;
			});
			content_entry.focus_in_event.connect ( () => {
				on_focus_in ();
				return false;
			});
			content_entry.key_release_event.connect ( (e) => {
				is_tag ();
				return false;
			});
			content_entry.button_release_event.connect ( (e) => {
				is_tag ();
				return false;
			});
			content_box.draw.connect ( (ctx) => {
				if (app.selected.note == this){
					ctx.rectangle (0, 0, content_entry.get_allocated_width (), 
					    content_entry.get_allocated_height ());
					ctx.set_source_rgba (0.0, 0.0, 0.0, 0.4);
					ctx.set_line_width (600.0);
					ctx.stroke ();
				}
				return false;
			});
			
			//editing bar
			editbar = new Toolbar ();
			editbar.margin_top = 1;
			editbar.get_style_context ().add_class ("inline-toolbar");
			editbar.set_icon_size (IconSize.MENU);
			
			var bold   = app.main_actions.get_action ("SetBold").create_tool_item () as ToolButton;
			var italic = app.main_actions.get_action ("SetItalic").create_tool_item () as ToolButton;
			var strike = app.main_actions.get_action ("StrikeThrough").create_tool_item () as ToolButton;
			var clear  = app.main_actions.get_action ("Clear").create_tool_item () as ToolButton;
			
			clear.icon_widget = new Image.from_icon_name ("edit-clear-symbolic", IconSize.MENU);
			
			editbar.insert (bold,        0);
			editbar.insert (italic,      1);
			editbar.insert (strike,      2);
			editbar.insert (clear,       3);
			
			editbar.show.connect ( () => {
				if (app.selected.note != this)
					editbar.hide ();
			});
			
			content_box.pack_start (editbar, false, true, 0);
			content_box.pack_start (content_entry);
			
			body.pack_start (header, false, true, 0);
			body.pack_start (content_box, true, false, 0);
			
			
			//tags
			content_entry.buffer.create_tag ("bold", "weight", Pango.Weight.BOLD);
			content_entry.buffer.create_tag ("italic", "style", Pango.Style.ITALIC);
			content_entry.buffer.create_tag ("strikethrough", "strikethrough", true);
			
			tag_area = new Footnote.Widgets.TaggingArea (this, app);
			for (var i=0;i<this.tags.size;i++){
				tag_area.add_tag(this.tags.get (i));
			}
			tag_area.tag_added.connect ( (name)=>{
				this.tags.add(name);
				this.save (app);
			});
			tag_area.tag_removed.connect ( (name)=>{
				this.tags.remove (name);
				this.save (app);
			});
			
			body.pack_start (tag_area);
			body.margin = 10;
			body.margin_top = 0;
			
			entry_box.pack_start (datelbl);
			entry_box.pack_start (body, true, false, 0);
			
			this.margin = 5;
			base.add (entry_box);
			
			tag_area.add_tag_entry.hide();
			
			TargetEntry e = {"footnote_note", 0, 0};
			Gtk.drag_source_set (this, Gdk.ModifierType.BUTTON1_MASK | 
			                           Gdk.ModifierType.BUTTON3_MASK, {e}, Gdk.DragAction.MOVE);
			this.drag_begin.connect ( (context) => {
				app.selected.note = this;
				app.selected.category = this.category; //just make sure...
			});
			app.selected.note = this;
			this.drag_data_get.connect ( (ctx, sel, info, time) => {
				sel.set (sel.get_target (), 8, this.id.to_string ().data);
			});
			
			
			this.button_release_event.connect ( (e) => {
				var x = (int)e.x;
				var y = (int)e.y;
				if ((x > 10) && (x < 42) && (y > 0) && (y < 32)){
					if (category.notes.size > 0) {
						app.selected.note = this;
						action_delete_note();
					}
				}else if ((x > this.width-32) && (x < this.width) && (y > 0) && (y < 32)){
					var dialog = new Dialog.with_buttons ("Export", app.mainwindow, DialogFlags.MODAL,
					     Stock.CANCEL, ResponseType.REJECT, Stock.OK, ResponseType.ACCEPT);
					var contr = new Granite.Widgets.ContractorView (save_temp_file (), "text");
					contr.margin = 12;
					((Box)dialog.get_content_area ()).add (contr);
					dialog.show_all ();
					if (dialog.run () == ResponseType.ACCEPT){
						contr.run_selected ();
					}
					dialog.destroy ();
				}
				return false;
			});
		}
		
 		public void collapse (){
            collapsed = true;
            this.category.cat_collapsed = true;
            view_in_main.show ();
 	        view_collapsed.hide ();
 	        datelbl.hide ();
     		content_box.hide ();
            tag_area.hide ();
        }
         
        public void _expand (){
    	    collapsed = false;
    	    datelbl.show ();
 	        content_box.show ();
    	    tag_area.show ();
            view_in_main.hide ();
        }
 			
 		public void modify (int force=0){
 		    if (force == 0){
 		        if (collapsed){
 	   	            _expand ();
 	   	            view_collapsed.show ();
 	   	        }else{
 	   	           collapse ();
                }
            }else if (force == 1){
                collapse ();
            }else{
  	           _expand ();
            }
        }
		
		public string save_temp_file (){
			var path = GLib.Environment.get_tmp_dir () + "/" + this.title_entry.buffer.text+".txt";
			print (path);
			try {
				GLib.FileUtils.set_contents (path, content_entry.buffer.text);
			}catch (GLib.Error e){warning (e.message);}
			return path;
		}
		
		public void apply_tag (string name){
			if (this.content_entry.buffer.has_selection){
				TextIter s; TextIter e;
				this.content_entry.buffer.get_selection_bounds (out s, out e);
				this.content_entry.buffer.apply_tag_by_name (name, s, e);
			}
		}
		public void is_tag (){
			/*TextIter s;
			if (content_entry.buffer.has_selection)
				content_entry.buffer.get_selection_bounds (out s, null);
			else
				content_entry.buffer.get_iter_at_offset (out s, content_entry.buffer.cursor_position);
			SList<TextTag> tags = s.get_tags ();
			app.prevent_apply = true;
			app.bold.active = false;
			app.prevent_apply = true;
			app.italic.active = false;
			app.prevent_apply = true;
			app.strike.active = false;
			for (var i=0;i<tags.length ();i++){
				app.prevent_apply = true;
				if (tags.nth_data (i).name == "bold")
					app.bold.active = true;
				if (tags.nth_data (i).name == "italic")
					app.italic.active = true;
				if (tags.nth_data (i).name == "strikethrough")
					app.strike.active = true;
			}*/
		}
		public void remove_tag (string name){
			/*if (this.content_entry.buffer.has_selection){
				TextIter s; TextIter e;
				this.content_entry.buffer.get_selection_bounds (out s, out e);
				this.content_entry.buffer.remove_tag_by_name (name, s, e);
			}*/
		}
		public void clear_tags (){
			if (this.content_entry.buffer.has_selection){
				TextIter s; TextIter e;
				this.content_entry.buffer.get_selection_bounds (out s, out e);
				this.content_entry.buffer.remove_all_tags (s, e);
			}
		}
		
		private void on_focus_in () {
			app.selected.note = this;
			this.tag_area.add_tag_entry.show ();
			this.editbar.show ();
			this.queue_draw ();
		}

		private void on_focus_out () {
			app.selected.note = null;
			this.save (app);
			this.editbar.hide ();
			this.tag_area.add_tag_entry.hide ();
			this.queue_draw ();
		}
		
		public Note.add (FootnoteApp app, Category category){
			this.category = category;
			this.tags = new ArrayList<string> ();
			this.app = app;
			var dt = new DateTime.now_local ();
			string[] weekdays = {_("Monday"),_("Tuesday"),_("Wednesday"),_("Thursday"),
			    _("Friday"),_("Saturday"),_("Sunday")};
			this.date = dt.get_hour ().to_string ()+":"+dt.get_minute ().to_string ()+" "+
			    weekdays[dt.get_day_of_week ()-1]+", "+dt.get_day_of_month ().to_string ()+"."+
			    dt.get_month ().to_string ()+"."+dt.get_year ().to_string ();
			this.build (app);
			
			this.month = dt.get_month ();
			this.year  = dt.get_year ();
			
			try {
				var q = new Query (app.db, "INSERT INTO 'notes' (title, content, category, tags, date) VALUES  (:title, :content, :category, :tags, :date);");
				q.set_string (":title", "");
				q.set_string (":content", "");
				q.set_int (":category", this.category.id);
				q.set_string (":tags", "");
				q.set_string (":date", this.date);
				q.execute ();
				this.id = (int)app.db.last_insert_id;
			}
			catch (SQLHeavy.Error err){stderr.printf ("Error: %s\n", err.message);}
			this.title = "";
			this.content = "";
			this.category = category;
			this.tags = new ArrayList<string> ();
			this.category.set_count_field (app, this.category.count+1);
		}
		
		public void save (FootnoteApp app){
			try{
				var tags_str = "";
				for (var i=0;i<tags.size;i++){
					tags_str +=  (string)tags.get (i)+",";
				}
				tags_str = tags_str.substring(0, tags_str.length-1);
				
				this.title = title_entry.buffer.text;
				//this.content_entry.buffer.register_serialize_tagset (null);
				this.content = content_entry.buffer.text;
				var q = app.db.prepare ("UPDATE 'notes' SET title=:title, content=:content, tags=:tags WHERE rowid=:id");
				q.set_string (":title", this.title);
				q.set_string (":content", this.content);
				q.set_int (":id", this.id);
				q.set_string (":tags", tags_str);
				q.execute ();
			}catch (SQLHeavy.Error e){stderr.printf ("Error: %s",e.message);}
		}
		
		public void del_from_db (FootnoteApp app){
			try{
				var q = new Query (app.db, "DELETE FROM 'notes' WHERE rowid = :id;");
				q.set_int (":id", this.id);
				q.execute ();
			}catch (SQLHeavy.Error e){stderr.printf ("Error: %s", e.message);}
		}
	}
}
	
