//
//  Copyright (C) 2011 Tom Beckmann
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

using Gee;
using Gtk;
using Gdk;

namespace Footnote.Widgets {
	
	/**
	 * Display a (filtered) list of Notes for the currently
	 * selected Category.
	 */
	public class NoteView : Gtk.VBox {
		
		public string filter;
		
		private FootnoteApp app;
		private Gtk.Label none_found;
				
		/**
		 * Constructor
		 */
		public NoteView (FootnoteApp app) {
			this.app = app;
			this.filter = "";
		
			this.none_found = new Gtk.Label("No notes found for this category");
			pack_start (this.none_found, true, true, 0);
			
			show_all ();
			none_found.hide ();
		}
		
		/**
		 * Add a note to the view
		 */
		public void add_note (Note n) {
			pack_start (n, false, false, 0);
			reorder_child (n, 0);
			update_view ();
		}
		
		/**
		 * Remove a note from the view
		 */
		public void remove_note (Note n) {
			remove (n);
			update_view ();
		}
		
		/**
		 * Update the view, loop through ArrayList of Notes and
		 * if filter isn't empty, only show notes containing the
		 * filter string (e.g. in title, text or as a tag).
		 */
		public void update_view () {
			var showed = 0;
			if (app.selected.category != null)
				foreach (Note n in app.selected.category.notes) {
					if (((app.month == -1) || (app.month == n.month)) && (app.year == n.year)){
						if (filter == "" || 
							filter.down () in n.title.down () || 
							filter.down () in n.content.down () || 
							n.tag_area.has_tag(filter)) {
							if (!n.visible)
								n.show_all ();
							showed++;
						}else{
							if (n.visible){
								n.hide ();
							}
						}
					}
					else{
						if (n.visible){
							n.hide ();
						}
					}
				}
			
			if (showed == 0){
				none_found.show ();
			}else{
				none_found.hide ();
			}
		}
	}

}
